{ **************************************************************************** }
{ *                                                                          * }
{ * GRAB MAN (unknown mfg, Game & Time) Simulator                            * }
{ * by Luca "MADrigal" Antignano, November 1999 (revision: August 2021)      * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit gfxinit;
{ this unit contains the graphics initialization (skin-based system)           }

interface

uses
  Forms, Controls, StdCtrls, SysUtils, Classes, Graphics, ExtCtrls, Dialogs,
  Jpeg, Pngimage, IniFiles;

{ * PROCEDURES: SKIN INITIALIZATION                                          * }
procedure Gfx_Game_Init;
procedure Gfx_Manual_Init;

const
  INIFILE = 'Config.ini';

{}IMAGES = 135;

{}DEFAULTHEIGHT    = 777;
{}DEFAULTWIDTH     = 479;
  DEFAULTSKINDIR   = 'Graphics\Skin\';
  DEFAULTWINMODE   = 'Shaped';

  DEFAULTTRCOLOR   = '$00ff00ff';

  DEFAULTBDDIR     = 'Graphics\Backdrop\';
  DEFAULTBDFILE    = 'nil';
  DEFAULTBDSTRETCH = 0;
{}DEFAULTBGCOLOR   = '$00000000';
  DEFAULTOFFSETX   = 'Center';
  DEFAULTOFFSETY   = 'Center';
  DEFAULTLOGOFILE  = 'nil';
  DEFAULTLOGOPOS   = 1;

  DEFAULTMANUALDIR = 'Graphics\Manual\';
{}DEFAULTMANHEIGHT = 535;
{}DEFAULTMANWIDTH  = 340;

var
  IF_File: TIniFile;
  aImages: array [1..IMAGES] of TImage;
{}//sVideoMode: string;

implementation

uses
  manual, splash, unit1;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: SKIN INITIALIZATION                                          * }
{ *                                                                          * }
{ **************************************************************************** }

procedure Gfx_Game_Init;
{ initializes the graphics on the main game form                               }
  var
    // service variables
    i,
    iOffsetX,
    iOffsetY: integer;

    // temporary variables (used to read/write from 'ini file')
    sSkinDir,
    sBackdropDir,
    sBackdropFile,
    sGameOffsetX,
    sGameOffsetY,
{}  sLogoFile: string;
{}//sWindowMode

    // image variables
    sFilename: string;
begin
  // initialises array of images
  with Form1 do
    begin
      aImages[1]  := im_background;              aImages[2]  := im_background_screen_top;

      aImages[3] := btn_mode_time;               aImages[4]  := btn_mode_top;
      aImages[5] := btn_ama_down;                aImages[6]  := btn_ama_top;
      aImages[7] := btn_pro_down;                aImages[8]  := btn_pro_top;
      aImages[9] := btn_time_down;               aImages[10] := btn_time_top;
                                                 aImages[11] := btn_set_top;
                                                 aImages[12] := btn_halt_top;

      aImages[13] := btn_up_down;                aImages[14] := btn_up_top;
      aImages[15] := btn_left_down;              aImages[16] := btn_left_top;
      aImages[17] := btn_right_down;             aImages[18] := btn_right_top;
      aImages[19] := btn_down_down;              aImages[20] := btn_down_top;

      aImages[21] := im_panel_digit_1; aImages[22] := im_panel_digit_2; aImages[23] := im_panel_digit_3; aImages[24] := im_panel_digit_4;
      aImages[25] := im_panel_digit_5; aImages[26] := im_panel_digit_6; aImages[27] := im_panel_digit_7; aImages[28] := im_panel_digit_8;

      aImages[29] := im_panel_number_0; aImages[30] := im_panel_number_1; aImages[31] := im_panel_number_2; aImages[32] := im_panel_number_3; aImages[33] := im_panel_number_4;
      aImages[34] := im_panel_number_5; aImages[35] := im_panel_number_6; aImages[36] := im_panel_number_7; aImages[37] := im_panel_number_8; aImages[38] := im_panel_number_9;

      aImages[39] := im_panel_letter_a; aImages[40] := im_panel_letter_d; aImages[41] := im_panel_letter_o; aImages[42] := im_panel_letter_p;

      aImages[43] := im_panel_lives_1; aImages[44] := im_panel_lives_2; aImages[45] := im_panel_lives_3;

      aImages[46] := im_panel_time_pm; aImages[47] := im_panel_over;

      aImages[48] := im_pacman_01_01; aImages[49] := im_pacman_01_02; aImages[50] := im_pacman_01_07; aImages[51] := im_pacman_01_08;
      aImages[52] := im_pacman_02_01; aImages[53] := im_pacman_02_02; aImages[54] := im_pacman_02_03; aImages[55] := im_pacman_02_04; aImages[56] := im_pacman_02_05; aImages[57] := im_pacman_02_06; aImages[58] := im_pacman_02_07; aImages[59] := im_pacman_02_08;
      aImages[60] := im_pacman_03_01; aImages[61] := im_pacman_03_02; aImages[62] := im_pacman_03_03; aImages[63] := im_pacman_03_06; aImages[64] := im_pacman_03_07; aImages[65] := im_pacman_03_08;
      aImages[66] := im_pacman_04_01; aImages[67] := im_pacman_04_02; aImages[68] := im_pacman_04_03; aImages[69] := im_pacman_04_04; aImages[70] := im_pacman_04_05; aImages[71] := im_pacman_04_06; aImages[72] := im_pacman_04_07; aImages[73] := im_pacman_04_08;
      aImages[74] := im_pacman_05_01; aImages[75] := im_pacman_05_02; aImages[76] := im_pacman_05_03; aImages[77] := im_pacman_05_04; aImages[78] := im_pacman_05_05; aImages[79] := im_pacman_05_06; aImages[80] := im_pacman_05_07; aImages[81] := im_pacman_05_08;

      aImages[82] := im_ghost_01_01;  aImages[83] := im_ghost_01_02;  aImages[84] := im_ghost_01_07;  aImages[85] := im_ghost_01_08;
      aImages[86] := im_ghost_02_01;  aImages[87] := im_ghost_02_02;  aImages[88] := im_ghost_02_03;  aImages[89] := im_ghost_02_04;  aImages[90] := im_ghost_02_05;  aImages[91] := im_ghost_02_06;  aImages[92] := im_ghost_02_07;  aImages[93] := im_ghost_02_08;
      aImages[94] := im_ghost_03_01;  aImages[95] := im_ghost_03_02;  aImages[96] := im_ghost_03_03;  aImages[97] := im_ghost_03_06;  aImages[98] := im_ghost_03_07;  aImages[99] := im_ghost_03_08;
      aImages[100] := im_ghost_04_01; aImages[101] := im_ghost_04_02; aImages[102] := im_ghost_04_03; aImages[103] := im_ghost_04_04; aImages[104] := im_ghost_04_05; aImages[105] := im_ghost_04_06; aImages[106] := im_ghost_04_07; aImages[107] := im_ghost_04_08;
      aImages[108] := im_ghost_05_01; aImages[109] := im_ghost_05_02; aImages[110] := im_ghost_05_03; aImages[111] := im_ghost_05_04; aImages[112] := im_ghost_05_05; aImages[113] := im_ghost_05_06; aImages[114] := im_ghost_05_07; aImages[115] := im_ghost_05_08;

      aImages[116] := im_ghost_pit_3; aImages[117] := im_ghost_pit_2; aImages[118] := im_ghost_pit_1;

      aImages[119] := im_pill_01_01_double; aImages[120] := im_pill_01_07_power;
      aImages[121] := im_pill_02_02_single; aImages[122] := im_pill_02_04_single; aImages[123] := im_pill_02_06_single; aImages[124] := im_pill_02_08_single;
      aImages[125] := im_pill_03_01_double; aImages[126] := im_cherry; aImages[127] := im_pill_03_07_double;
      aImages[128] := im_pill_04_02_single; aImages[129] := im_pill_04_04_single; aImages[130] := im_pill_04_06_single; aImages[131] := im_pill_04_08_single;
      aImages[132] := im_pill_05_01_power;  aImages[133] := im_pill_05_03_single; aImages[134] := im_pill_05_05_single; aImages[135] := im_pill_05_07_double;
    end;

  // TO BE USED TO GENERATE THE SKIN FILE ONLY (TO BE COMMENTED THEREAFTER)
{  IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'Export.ini');

  for i := 1 to IMAGES do
    begin
      if aImages[i].Picture.Graphic = nil then
        sFilename := 'nil'
      else
        if aImages[i].Picture.Graphic is TJPEGImage then
          sFilename := aImages[i].Name + '.jpg'
        else
          if aImages[i].Picture.Graphic is TBitmap then
            sFilename := aImages[i].Name + '.bmp'
          else
            if aImages[i].Picture.Graphic is TPNGObject then
              sFilename := aImages[i].Name + '.png';

      IF_file.WriteString('Image' + IntToStr(i),  'Filename', sFilename);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Height',   aImages[i].Height);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Left',     aImages[i].Left);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Top',      aImages[i].Top);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Width',    aImages[i].Width);
    end;

  IF_File.Destroy; }

  // loads configuration settings from the Config.ini file
  if not FileExists(ExtractFilePath(ParamStr(0)) + INIFILE) then
    begin
      MessageDlg('Configuration file (Config.ini) not found. Please reinstall the game.', mtError, [mbOK], 0);
      Application.Terminate;
    end
  else
    begin
      // reads init values from 'ini file' -- with defaults if no 'ini file' found
      IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + INIFILE);

      // game form initialization (game graphics and backdrop)
      with Form1 do
        begin
          sSkinDir     := IncludeTrailingPathDelimiter(IF_file.ReadString('Common', 'SkinDir', DEFAULTSKINDIR));
          ClientHeight := IF_file.ReadInteger('Common', 'GameHeight', DEFAULTHEIGHT);
          ClientWidth  := IF_file.ReadInteger('Common', 'GameWidth',  DEFAULTWIDTH);

          // use Fullscreen or Shaped mode
{}        if bFullscreenMode then
            begin                                                               // Fullscreen
              // sets backdrop image and background colour
              sBackdropDir  := IncludeTrailingPathDelimiter(IF_file.ReadString('Fullscreen', 'BackdropDir',  DEFAULTBDDIR));
              sBackdropFile := IF_file.ReadString('Fullscreen', 'BackdropFile', DEFAULTBDFILE);

              if (sBackdropFile <> '')
                and (sBackdropFile <> 'nil') then
                if FileExists(sBackdropDir + sBackdropFile) then
                  begin
                    im_backdrop.Picture.LoadFromFile(sBackdropDir + sBackdropFile);
                    im_backdrop.Stretch := Boolean(IF_file.ReadInteger('Fullscreen', 'BackdropStretch', DEFAULTBDSTRETCH));
{}                  im_backdrop.Visible := true;
                  end;

              Color := StrToInt(IF_file.ReadString('Fullscreen', 'BgColor', DEFAULTBGCOLOR));
              TransparentColor := false;

              // sets new coordinates for the handheld game graphics
              sGameOffsetX := IF_file.ReadString('Fullscreen', 'GameOffsetX', DEFAULTOFFSETX);
              sGameOffsetY := IF_file.ReadString('Fullscreen', 'GameOffsetY', DEFAULTOFFSETY);

              if StrToIntDef(sGameOffsetX, 0) <> 0 then iOffsetX := StrToInt(sGameOffsetX) else iOffsetX := Trunc((Screen.Width  - ClientWidth)  / 2);
              if StrToIntDef(sGameOffsetY, 0) <> 0 then iOffsetY := StrToInt(sGameOffsetY) else iOffsetY := Trunc((Screen.Height - ClientHeight) / 2);

              // sets the form as full screen
              ClientWidth  := Screen.Width;
              ClientHeight := Screen.Height;

              // sets logo (watermark) image and its position
              sLogoFile := IF_file.ReadString('Fullscreen', 'LogoFile', DEFAULTLOGOFILE);

              if (sLogoFile <> '')
                and (sLogoFile <> 'nil') then
                if FileExists(sBackdropDir + sLogoFile) then
                  with Form1 do
                    begin
                      im_backdrop_logo.Picture.LoadFromFile(sBackdropDir + sLogoFile);
                      im_backdrop_logo.AutoSize := true;
{}                    im_backdrop_logo.Visible := true;

                      case IF_file.ReadInteger('Fullscreen', 'LogoPosition', DEFAULTLOGOPOS) of
                        0: begin im_backdrop_logo.Left := 0;                                    im_backdrop_logo.Top := 0; end; // top-left
                        1: begin im_backdrop_logo.Left := ClientWidth - im_backdrop_logo.Width; im_backdrop_logo.Top := 0; end; // top-right
                        2: begin im_backdrop_logo.Left := 0;                                    im_backdrop_logo.Top := ClientHeight - im_backdrop_logo.Height; end; // bottom-left
                        3: begin im_backdrop_logo.Left := ClientWidth - im_backdrop_logo.Width; im_backdrop_logo.Top := ClientHeight - im_backdrop_logo.Height; end; // bottom-right
                      end; // case..of
                    end;
            end
          else                                                                  // Shaped
            begin
              // sets coordinates for the handheld game graphics
              iOffsetX := 0;
              iOffsetY := 0;

              // sets the form as 'shaped'
              Color                 := StrToInt(IF_file.ReadString('VideoMode', 'TranspColor', DEFAULTTRCOLOR));
              TransparentColorValue := StrToInt(IF_file.ReadString('VideoMode', 'TranspColor', DEFAULTTRCOLOR));
              TransparentColor      := true;

              // disables the backdrop
              im_backdrop.Visible := false;
              im_backdrop_logo.Visible := false;
            end;
        end;

      // game skin initialization
      for i := 1 to IMAGES do
        begin
          sFilename := IF_file.ReadString('Image' + IntToStr(i), 'Filename', '');

          if (sFilename <> '')
            and (sFilename <> 'nil') then
            if FileExists(sSkinDir + sFilename) then
              // loads picture into the image
              aImages[i].Picture.LoadFromFile(sSkinDir + sFilename)
            else
              // image expected but not found -> creates a coloured square as a placeholder
              begin
                aImages[i].Picture.Bitmap.Create;
                aImages[i].Picture.Bitmap.Width  := 10;
                aImages[i].Picture.Bitmap.Height := 10;
                aImages[i].Picture.Bitmap.Canvas.Create;
                aImages[i].Picture.Bitmap.Canvas.Pen.Color := 0;
                aImages[i].Picture.Bitmap.Canvas.Pen.Style := psSolid;
                aImages[i].Picture.Bitmap.Canvas.Brush.Color := $000000ff;
                aImages[i].Picture.Bitmap.Canvas.Rectangle(0, 0, 10, 10);
              end;

          aImages[i].Height := IF_file.ReadInteger('Image' + IntToStr(i), 'Height', 0);
          aImages[i].Left   := IF_file.ReadInteger('Image' + IntToStr(i), 'Left',   0) + iOffsetX;
          aImages[i].Top    := IF_file.ReadInteger('Image' + IntToStr(i), 'Top',    0) + iOffsetY;
          aImages[i].Width  := IF_file.ReadInteger('Image' + IntToStr(i), 'Width',  0);
        end;

      // close Inifile
      IF_File.Destroy;
    end;

  // initializes the on-screen game menu
  with Form1 do
    begin
{}    btn_game_menu_top.Visible := bFullscreenMode;
{}    btn_game_menu_background.Visible := bFullscreenMode;

      btn_game_menu_background.Height := 32;
      btn_game_menu_background.Width := 36;
    end;
end;





procedure Gfx_Manual_Init;
{ initializes the graphics of the instructions manual form                     }
  var
    // service variables
    i: integer;

    // temporary variables (used to read/write from 'ini file')
    sManualDir: string;

    // image variables
    sFilename: string;
begin
  // reads init values from 'ini file' -- with defaults if no 'ini file' found
  IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + INIFILE);

  // manual form initialization (graphics and sizes)
  with ManualForm do
    begin
      sManualDir   := IncludeTrailingPathDelimiter(IF_file.ReadString('Manual', 'ManualDir', DEFAULTMANUALDIR));

      ClientHeight := IF_file.ReadInteger('Manual', 'ManualHeight', DEFAULTMANHEIGHT);
      ClientWidth  := IF_file.ReadInteger('Manual', 'ManualWidth',  DEFAULTMANWIDTH);

      im_page_top.Left   := 0;
      im_page_top.Top    := 0;
      im_page_top.Height := ClientHeight;
      im_page_top.Width  := ClientWidth;

      sh_page_top.Left   := 0;
      sh_page_top.Top    := 0;
      sh_page_top.Height := ClientHeight;
      sh_page_top.Width  := ClientWidth;
    end;

  // instructions manual skin initialization
  for i := 1 to MANUALPAGES do
    begin
      sFilename := IntToStr(i) + '.bmp';

      if FileExists(sManualDir + sFilename) then
          // loads picture into the image
          aIm_pages[i].Picture.LoadFromFile(sManualDir + sFilename)
        else
          // image expected but not found -> creates a coloured square as a placeholder
          begin
            aIm_pages[i].Picture.Bitmap.Create;
            aIm_pages[i].Picture.Bitmap.Width  := 10;
            aIm_pages[i].Picture.Bitmap.Height := 10;
            aIm_pages[i].Picture.Bitmap.Canvas.Create;
            aIm_pages[i].Picture.Bitmap.Canvas.Pen.Color := 0;
            aIm_pages[i].Picture.Bitmap.Canvas.Pen.Style := psSolid;
            aIm_pages[i].Picture.Bitmap.Canvas.Brush.Color := $000000ff;
            aIm_pages[i].Picture.Bitmap.Canvas.Rectangle(0, 0, 10, 10);
          end;

      with ManualForm do
        begin
          aIm_pages[i].Height := ClientHeight;
          aIm_pages[i].Width  := ClientWidth;
        end;
    end;

  // close Inifile
  IF_File.Destroy;
end;





end.
